#ifndef DGX_RENDER_3D_H
#define DGX_RENDER_3D_H

#include <engine/psx.h>

#define WF_WHITE_BUF_SIZE	1000

#define MAX_SUBDIV		6
#define MAX_VERTS		500
#define MAX_OTZBUF		450

#define NUM_LOD			2
#define LOD_HI 			0
#define LOD_LO 			1

#define NUMLIGHTS		3

#define ROTM_XYZ		0
#define ROTM_YXZ		1
#define ROTM_ZYX		2

#define MAX_UNIQUE_POLY		8

#define IDX_F3			0
#define IDX_F4			1
#define IDX_FT3			2
#define IDX_FT4			3
#define IDX_G3			4
#define IDX_G4			5
#define IDX_GT3			6
#define IDX_GT4			7

/*
 * <asset>_add flag
 *	e.g. mtd_add, atd_add, etc..
 *
 *	XADD_USE_PIVOT_LOCAL2 ensures true model-space
 *	rotation in "submodels" that are associated
 *	with a main 3d model. (e.g. a helicopter's
 *	tail rotor)
 *
 *	XADD_USE_PIVOT_LOCAL3 completely rewrites
 *	the original model coords (p_vertex) with new
 *	ones. Use with care.
 */
#define XADD_CONVEX_CLIP	(1<<0)	/* rotavg bits 0-15 */
#define XADD_CONVEX_CLIP_B	(1<<1)
#define XADD_DEPTH_CUE		(1<<2)
#define XADD_USE_OT		(1<<16) /* local bits 16-31 */
#define XADD_SUBDIV		(1<<17)
#define XADD_USE_PIVOT_LOCAL	(1<<18) /* DEPRECATED - enabled by default unless XADD_NO_SRT_R is specified */
#define XADD_USE_PIVOT_LOCAL2	(1<<19)
#define XADD_USE_PIVOT_LOCAL3	(1<<20)
#define XADD_NO_SRT_S		(1<<21)
#define XADD_NO_SRT_R		(1<<22)
#define XADD_NO_SRT_T		(1<<23)
#define XADD_SAVE_OTZ		(1<<24)

/* vert_srt flag */
#define SRT_S		(1<<0)
#define SRT_R		(1<<1)
#define SRT_T		(1<<2)

/*
 * dgx_light_flag:
 *	bits 0-2:	active light sources (sun)
 *	bits 3-5:	active light sources (point)
 *	bits 3-31:	unused
 */
int		dgx_light_flag;

/*
 * dgx_light_sun:
 *	rows: indices
 *	cols: x,y,z
 *
 * dgx_light_sun_color:
 *	rows: r,g,b
 *	cols: indices
 */

MATRIX		dgx_light_sun;
MATRIX		dgx_light_sun_color;
SVECTOR		dgx_light_sun_ang[NUMLIGHTS];

VECTOR		dgx_light_point_pos[NUMLIGHTS];
int		dgx_light_point_radius[NUMLIGHTS];
CVECTOR		dgx_light_point_color[NUMLIGHTS];

int rotavg3();
int rotavg4();

#endif

